import React from 'react';
import { CompositeLayer } from 'deck.gl';
import { ArcLayer, GeoJsonLayer, ScatterplotLayer } from '@deck.gl/layers';
import { netherlands } from '../../../../../../mock-data/geo-json';
import { Edge, Node, LayerProps } from '../../../mapvis.types';
import { RGBColor, color, geoBounds, geoCentroid, geoContains, interpolateYlOrRd, scaleSequential } from 'd3';

export class ChoroplethLayer extends CompositeLayer<LayerProps> {
  static type = 'Choropleth';

  shouldUpdateState({ props, oldProps, context, changeFlags }: { props: any; oldProps: any; context: any; changeFlags: any }) {
    return changeFlags.dataChanged || changeFlags.propsChanged;
  }

  updateState({ props, oldProps, context, changeFlags }: { props: any; oldProps: any; context: any; changeFlags: any }) {
    if (changeFlags.dataChanged) {
      const nodes = props.graph.nodes;
      const geojson = netherlands;
      const data = this.concatData(geojson, nodes);
      this.setState({ data });
    }
  }

  concatData(geojson: any, nodes: Node[]) {
    const updatedGeojson = { ...geojson };

    nodes.map((node: Node) => {
      const coordinates: [number, number] = this.props.getNodeLocation(node.id);

      if (coordinates) {
        updatedGeojson.features.map((feature: any) => {
          const isInside = geoContains(feature, coordinates);
          if (isInside) {
            feature.properties.nodes = feature.properties.nodes ?? [];
            feature.properties.nodes.push(node.id);

            feature.properties.townships.map((township: any) => {
              const isInside = geoContains(township, coordinates);

              if (isInside) {
                township.properties.nodes = township.properties.nodes ?? [];
                township.properties.nodes.push(node.id);
              }
            });
            return;
          }
        });
      }
    });

    return updatedGeojson;
  }

  getColor(polygon: any, min: number, max: number) {
    const nodeLength = polygon.properties?.nodes?.length ?? 0;

    if (nodeLength === min) return [128, 128, 128];

    const colorScale = scaleSequential(interpolateYlOrRd).domain([min, max]);
    const _color = color(colorScale(nodeLength)) as RGBColor;

    return _color ? [_color.r, _color.g, _color.b] : [100, 0, 0];
  }

  renderLayers() {
    const { data } = this.state;

    const layers: any = [];

    let max = 0;

    (data as any).features.forEach((feature: any) => {
      const nodeCount = feature.properties?.nodes ? feature.properties.nodes.length : 0;
      if (nodeCount > max) max = nodeCount;
    });

    (data as any).features.forEach((feature: any) => {
      layers.push(
        new GeoJsonLayer(
          this.getSubLayerProps({
            id: `feature-layer-${feature.properties.name}`,
            data: feature,
            opacity: 0.3,
            pickable: true,
            stroked: true,
            filled: true,
            extruded: false,
            lineWidthScale: 0.5,
            lineWidthMinPixels: 1,
            getLineWidth: (d: any) => 1,
            getLineColor: (d: any) => [220, 220, 220],
            getFillColor: (d: any) => this.getColor(d, 0, max),
          }),
        ),
      );
    });

    return [...layers];
  }
}

ChoroplethLayer.layerName = 'Choropleth';
